@@ -7,7 +7,7 @@ use std::env;
 use std::fmt::Write as FmtWrite;
 use std::fs::{self, File};
 use std::io::{self, BufRead, BufReader, Write};
-use std::path::PathBuf;
+use std::path::{Path, PathBuf};
 use std::process::Command;
 use std::str;
 
@@ -187,7 +187,7 @@ fn switch(configure: &mut Command, feature: &str, name: &str) {
     configure.arg(arg.to_string() + name);
 }
 
-fn get_ffmpet_target_os() -> String {
+fn get_ffmpeg_target_os() -> String {
     let cargo_target_os = env::var("CARGO_CFG_TARGET_OS").unwrap();
     match cargo_target_os.as_str() {
         "ios" => "darwin".to_string(),
@@ -195,7 +195,53 @@ fn get_ffmpet_target_os() -> String {
     }
 }
 
-fn build() -> io::Result<()> {
+/// Find the sysroot required for a cross compilation by ffmpeg **and** bindgen
+/// @see https://github.com/rust-lang/rust-bindgen/issues/1229
+fn find_sysroot() -> Option<String> {
+    if env::var("CARGO_FEATURE_BUILD").is_err() || env::var("HOST") == env::var("TARGET") {
+        return None;
+    }
+
+    if let Ok(sysroot) = env::var("SYSROOT") {
+        return Some(sysroot.to_string());
+    }
+
+    if env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("ios") {
+        let xcode_output = Command::new("xcrun")
+            .args(["--sdk", "iphoneos", "--show-sdk-path"])
+            .output()
+            .expect("failed to run xcrun");
+
+        if !xcode_output.status.success() {
+            panic!("Failed to run xcrun to get the ios sysroot, please install xcode tools or provide sysroot using $SYSROOT env. Error: {}", String::from_utf8_lossy(&xcode_output.stderr));
+        }
+
+        let string = String::from_utf8(xcode_output.stdout)
+            .expect("Failed to parse xcrun output")
+            .replace("\n", "");
+
+        if !Path::new(&string).exists() {
+            panic!("xcrun returned invalid sysroot path: {}", string);
+        }
+
+        return Some(string);
+    }
+
+    if env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("android") {
+        let sysroot_path = env::var("CARGO_NDK_SYSROOT_PATH").expect("Missing android sysroot path. For android cross compilation please use cargo-ndk which exposes all the required NDK paths throught env variables.");
+
+        if !Path::new(&sysroot_path).exists() {
+            panic!("Android sysroot path does not exists: {}", sysroot_path);
+        }
+
+        return Some(sysroot_path);
+    }
+
+    println!("cargo:warning=Detected cross compilation but sysroot not provided");
+    None
+}
+
+fn build(sysroot: Option<&str>) -> io::Result<()> {
     let source_dir = source();
 
     // Command's path is not relative to command's current_dir
@@ -223,17 +269,88 @@ fn build() -> io::Result<()> {
             configure.arg(format!("--extra-ldflags={}", target_flag));
         }
 
-        let compiler = cc.get_compiler();
-        let compiler = compiler.path().file_stem().unwrap().to_str().unwrap();
-        if let Some(suffix_pos) = compiler.rfind('-') {
-            let prefix = compiler[0..suffix_pos].trim_end_matches("-wr"); // "wr-c++" compiler
-            configure.arg(format!("--cross-prefix={}-", prefix));
-        }
         configure.arg(format!(
             "--arch={}",
             env::var("CARGO_CFG_TARGET_ARCH").unwrap()
         ));
-        configure.arg(format!("--target_os={}", get_ffmpet_target_os()));
+        configure.arg(format!("--target-os={}", get_ffmpeg_target_os()));
+
+        // cross-prefix won't work for android because they use different compiler for every
+        // platform version, so we provide direct compiler paths manually instead
+        if env::var("CARGO_CFG_TARGET_OS").as_deref() != Ok("android") {
+            let compiler = cc.get_compiler();
+            let compiler = compiler.path().file_stem().unwrap().to_str().unwrap();
+
+            if let Some(suffix_pos) = compiler.rfind('-') {
+                let prefix = compiler[0..suffix_pos].trim_end_matches("-wr"); // "wr-c++" compiler
+
+                configure.arg(format!("--cross-prefix={}-", prefix));
+            }
+        }
+    } else {
+        // tune the compiler for the host arhitecture
+        configure.arg("--extra-cflags=-march=native -mtune=native");
+    }
+
+    // for ios it is required to provide sysroot for both configure and bindgen
+    // for macos the easiest way is to run xcrun, for other platform we support $SYSROOT var
+    if env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("ios") {
+        let sysroot = sysroot.expect("The sysroot is required for ios cross compilation, make sure to have available xcode or provide the $SYSROOT env var");
+        configure.arg(format!("--sysroot={}", sysroot));
+
+        let cc = Command::new("xcrun")
+            .args(["--sdk", "iphoneos", "-f", "clang"])
+            .output()
+            .expect("failed to run xcrun")
+            .stdout;
+
+        configure.arg(format!(
+            "--cc={}",
+            str::from_utf8(&cc)
+                .expect("Failed to parse xcrun output")
+                .trim()
+        ));
+    }
+
+    if env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("android") {
+        // cargo ndk auto populates rust env variables for android cross compilation
+        // so we can just leverage the same compiler path and cflags for ffmpeg build
+        let android_cc_raw_path = env::var(format!("CC_{}", target)).expect("Missing CC path for android. Make sure to use cargo-ndk for adnrdoic cross compilation");
+        let android_cc_path = Path::new(&android_cc_raw_path);
+        if !android_cc_path.exists() {
+            panic!("Android CC path does not exists: {}", android_cc_raw_path);
+        }
+        configure.arg(format!("--cc={android_cc_raw_path}"));
+
+        for tool in ["nm", "strip"] {
+            configure.arg(format!(
+                "--{tool}={}",
+                android_cc_path
+                    .join("..")
+                    .join(format!("llvm-{tool}"))
+                    .canonicalize()
+                    .unwrap_or_else(|_| panic!("failed to resolve a path to android {}", tool))
+                    .display()
+            ));
+        }
+
+        if let Ok(android_target_flags) = env::var(format!("CFLAGS_{}", target)).as_deref() {
+            configure.arg(format!("--extra-cflags={android_target_flags}"));
+            configure.arg(format!("--extra-ldflags={android_target_flags}"));
+        }
+
+        if matches!(
+            env::var("CARGO_CFG_TARGET_ARCH").as_deref(),
+            Ok("x86_64") | Ok("x86")
+        ) {
+            // x86 asm contains position dependent code (relocations)
+            configure.arg("--disable-asm");
+        }
+
+        // https://github.com/Javernaut/ffmpeg-android-maker/blob/master/scripts/ffmpeg/build.sh#L30
+        // configure.arg(--extra-ldflags=-WL,-z,max-page-size=16384");
+        // required for android
+        configure.arg("--extra-cflags=-fPIC");
     }
 
     // control debug build
@@ -243,6 +360,9 @@ fn build() -> io::Result<()> {
     } else {
         configure.arg("--disable-debug");
         configure.arg("--enable-stripping");
+        configure.arg("--extra-cflags=-03 -ffast-math -funroll-loops");
+        #[cfg(not(target_os = "windows"))]
+        configure.arg("--extra-ldflags=-flto");
     }
 
     // make it static
@@ -250,6 +370,7 @@ fn build() -> io::Result<()> {
     configure.arg("--disable-shared");
     configure.arg("--enable-pthreads");
 
+    // position independent code
     configure.arg("--enable-pic");
 
     // stop autodetected libraries enabling themselves, causing linking errors
@@ -258,6 +379,9 @@ fn build() -> io::Result<()> {
     // do not build programs since we don't need them
     configure.arg("--disable-programs");
 
+    // do not generate documentation
+    configure.arg("--disable-doc");
+
     macro_rules! enable {
         ($conf:expr, $feat:expr, $name:expr) => {
             if env::var(concat!("CARGO_FEATURE_", $feat)).is_ok() {
@@ -348,6 +472,109 @@ fn build() -> io::Result<()> {
     enable!(configure, "BUILD_LIB_AVS", "libavs");
     enable!(configure, "BUILD_LIB_XVID", "libxvid");
 
+    // make sure to only enable related hw acceleration features for a correct
+    // target os. This allows to leave allows cargo features enable and control
+    // ffmpeg compilation using target only
+    let target_os = env::var("CARGO_CFG_TARGET_OS").unwrap();
+
+    // Apple VideoToolbox (iOS and macOS)
+    if env::var("CARGO_FEATURE_BUILD_VIDEOTOOLBOX").is_ok()
+        && matches!(target_os.as_str(), "ios" | "macos")
+    {
+        configure.arg("--enable-videotoolbox");
+
+        if target != host && env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("ios") {
+            configure.arg("--extra-cflags=-mios-version-min=11.0");
+        }
+
+        if target != host && env::var("CARGO_CFG_TARGET_OS").as_deref() == Ok("macos") {
+            configure.arg("--extra-cflags=-mmacosx-version-min=10.11");
+        }
+    }
+
+    // Apple audio hw acceleration API (iOS and macOS)
+    if env::var("CARGO_FEATURE_BUILD_AUDIOTOOLBOX").is_ok()
+        && matches!(target_os.as_str(), "ios" | "macos")
+    {
+        configure.arg("--enable-audiotoolbox");
+        configure.arg("--extra-cflags=-mios-version-min=11.0");
+    }
+
+    // Linux video acceleration API (VAAPI)
+    if env::var("CARGO_FEATURE_BUILD_VAAPI").is_ok() && matches!(target_os.as_str(), "linux") {
+        configure.arg("--enable-vaapi");
+    }
+
+    if env::var("CARGO_FEATURE_BUILD_LIB_D3D11VA").is_ok()
+        && matches!(target_os.as_str(), "windows")
+    {
+        configure.arg("--enable-d3d11va");
+    }
+
+    // DirectX Video Acceleration 2 (Windows only)
+    if env::var("CARGO_FEATURE_BUILD_LIB_DXVA2").is_ok() && matches!(target_os.as_str(), "windows")
+    {
+        configure.arg("--enable-dxva2");
+    }
+
+    // NVIDIA NVENC/NVDEC acceleration (Linux and Windows)
+    if env::var("CARGO_FEATURE_BUILD_NVIDIA").is_ok()
+        && matches!(target_os.as_str(), "linux" | "windows")
+    {
+        configure.arg("--enable-libnpp");
+        configure.arg("--enable-cuda-nvcc");
+        configure.arg("--enable-cuvid");
+        configure.arg("--enable-nvenc");
+        configure.arg("--enable-cuda-llvm");
+
+        let cuda_path = env::var("CUDA_PATH").unwrap_or(if target_os == "linux" {
+            "/usr/local/cuda".to_string()
+        } else {
+            "C:\\Program Files\\NVIDIA GPU Computing Toolkit\\CUDA".to_string()
+        });
+        println!("cargo:rustc-link-arg=-L{cuda_path}/lib64");
+        println!("cargo:rustc-link-arg=-I{cuda_path}/include");
+
+        // Additional configuration may be needed for CUDA toolkit path
+        // This could be provided as an environment variable
+        if let Ok(cuda_path) = env::var("CUDA_PATH") {
+            configure.arg(format!("--cuda-path={}", cuda_path));
+        }
+    }
+
+    // Intel Quick Sync Video (Linux and Windows)
+    if env::var("CARGO_FEATURE_BUILD_LIB_LIBMFX").is_ok()
+        && matches!(target_os.as_str(), "linux" | "windows")
+    {
+        configure.arg("--enable-libmfx");
+    }
+
+    // Android MediaCodec (Android only)
+    if env::var("CARGO_FEATURE_BUILD_MEDIACODEC").is_ok() && target_os == "android" {
+        configure.arg("--enable-mediacodec");
+        configure.arg("--enable-jni");
+
+        // Add common MediaCodec decoders
+        configure.arg("--enable-decoder=h264_mediacodec");
+        configure.arg("--enable-decoder=hevc_mediacodec");
+        configure.arg("--enable-decoder=vp8_mediacodec");
+        configure.arg("--enable-decoder=vp9_mediacodec");
+        configure.arg("--enable-decoder=av1_mediacodec");
+    }
+
+    // AMD Advanced Media Framework (Linux and Windows)
+    if env::var("CARGO_FEATURE_BUILD_AMF").is_ok()
+        && matches!(target_os.as_str(), "linux" | "windows")
+        && matches!(
+            env::var("CARGO_FEATURE_TARGET_ARCH").as_deref(),
+            Ok("x86_64")
+        )
+    {
+        configure.arg("--enable-amf");
+    }
+
+    enable!(configure, "BUILD_VULKAN", "vulkan");
+
     // other external libraries
     enable!(configure, "BUILD_LIB_DRM", "libdrm");
     enable!(configure, "BUILD_NVENC", "nvenc");
@@ -359,12 +586,23 @@ fn build() -> io::Result<()> {
     // configure misc build options
     enable!(configure, "BUILD_PIC", "pic");
 
+    // skip all the warnings from output as they can significantly slow down the build
+    // time on platforms like mac which spawns thousands of nullabilty complieance warnings
+    configure.arg("--extra-cflags=-w");
+
     // run ./configure
     let output = configure
         .output()
         .unwrap_or_else(|_| panic!("{:?} failed", configure));
     if !output.status.success() {
-        println!("configure: {}", String::from_utf8_lossy(&output.stdout));
+        println!(
+            "configure stdout: {}",
+            String::from_utf8_lossy(&output.stdout)
+        );
+        println!(
+            "configure stderr: {}",
+            String::from_utf8_lossy(&output.stderr)
+        );
 
         return Err(io::Error::new(
             io::ErrorKind::Other,
@@ -662,6 +900,7 @@ fn link_to_libraries(statik: bool) {
             println!("cargo:rustc-link-lib={}={}", ffmpeg_ty, lib.name);
         }
     }
+    println!("cargo:rustc-link-arg=-Wl,--no-as-needed");
     if env::var("CARGO_FEATURE_BUILD_ZLIB").is_ok() && cfg!(target_os = "linux") {
         println!("cargo:rustc-link-lib=z");
     }
@@ -671,6 +910,7 @@ fn main() {
     let statik = env::var("CARGO_FEATURE_STATIC").is_ok();
     let ffmpeg_major_version: u32 = env!("CARGO_PKG_VERSION_MAJOR").parse().unwrap();
 
+    let sysroot = find_sysroot();
     let include_paths: Vec<PathBuf> = if env::var("CARGO_FEATURE_BUILD").is_ok() {
         println!(
             "cargo:rustc-link-search=native={}",
@@ -680,7 +920,7 @@ fn main() {
         if fs::metadata(search().join("lib").join("libavutil.a")).is_err() {
             fs::create_dir_all(output()).expect("failed to create build directory");
             fetch().unwrap();
-            build().unwrap();
+            build(sysroot.as_deref()).unwrap();
         }
 
         // Check additional required libraries.
@@ -696,7 +936,7 @@ fn main() {
                     if line.starts_with("EXTRALIBS") {
                         Some(
                             line.split('=')
-                                .last()
+                                .next_back()
                                 .unwrap()
                                 .split(' ')
                                 .map(|s| s.to_string())
@@ -780,7 +1020,7 @@ fn main() {
                 println!("cargo:rustc-link-lib=ws2_32");
             }
 
-            // avutil depdendencies
+            // avutil dependencies
             println!("cargo:rustc-link-lib=bcrypt");
             println!("cargo:rustc-link-lib=user32");
         }
@@ -821,7 +1061,12 @@ fn main() {
             .include_paths
     };
 
-    if statik && cfg!(target_os = "macos") {
+    if statik
+        && matches!(
+            env::var("CARGO_CFG_TARGET_OS").as_deref(),
+            Ok("macos") | Ok("ios")
+        )
+    {
         let frameworks = vec![
             "AppKit",
             "AudioToolbox",
@@ -1253,6 +1498,10 @@ fn main() {
         .size_t_is_usize(true)
         .parse_callbacks(Box::new(Callbacks));
 
+    if let Some(sysroot) = sysroot.as_deref() {
+        builder = builder.clang_arg(format!("--sysroot={}", sysroot));
+    }
+
     // The input headers we would like to generate
     // bindings for.
     if env::var("CARGO_FEATURE_AVCODEC").is_ok() {
